#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec  6 12:39:30 2023

@author: anton
"""

from SLiCAP import *
from PyLTSpice import Trace, RawRead, SpiceEditor, SimRunner
from noise2 import Ri
from noiseWLI_p import W_finger, Mf, Le, f_min, f_max, ID # ID for minimum noise
from oneStage_p import IDp                                # IDp for drive capability

print("\n**********************************************************************")
print("* Check the AC performance of single CS stage with LTspice           *")
print("**********************************************************************\n")

netlist = SpiceEditor('cir/oneStageLTspice_p.cir')
netlist.set_parameters(Lp=Le, Wp=W_finger, Mp=Mf, fmin=f_min, fmax=f_max, R_i=Ri, ID=IDp)
#netlist.add_instructions(".noise V(out) V4 dec 20 {fmin} {fmax}")
#netlist.add_instructions(".tran 0 10m 9m")
netlist.add_instructions(".ac dec 20 10 100k")

fi = open('cir/oneStageLTspice_ac.cir', 'w')
fi.writelines(netlist.netlist)
fi.close()

runLTspice('cir/oneStageLTspice_ac.cir')

LTC                      = SimRunner(output_folder = './cir')
LTR                      = RawRead('cir/oneStageLTspice_ac.raw')
LTspiceACFrequencies     = LTR.get_trace('frequency').data
LTspiceAC                = np.abs(LTR.get_trace('V(out)').data)
LTspiceTrace1            = trace([LTspiceACFrequencies, LTspiceAC])
LTspiceTrace1.label      = 'LTspice: A1 P anti-series CS Stage'

print("\n**********************************************************************")
print("* Check the AC performance of single cascode stage with LTspice      *")
print("**********************************************************************\n")

netlist = SpiceEditor('cir/oneCascodeStageLTspice_p.cir')
netlist.set_parameters(Lp=Le, Wp=W_finger, Mp=Mf, fmin=f_min, fmax=f_max, R_i=Ri, ID=IDp)
netlist.add_instructions(".ac dec 20 10 100k")

fi = open('cir/oneCascodeStageLTspice_ac.cir', 'w')
fi.writelines(netlist.netlist)
fi.close()

runLTspice('cir/oneCascodeStageLTspice_ac.cir')

LTC                      = SimRunner(output_folder = './cir')
LTR                      = RawRead('cir/oneCascodeStageLTspice_ac.raw')
LTspiceACFrequencies     = LTR.get_trace('frequency').data
LTspiceAC                = np.abs(LTR.get_trace('V(out)').data)
LTspiceTrace2             = trace([LTspiceACFrequencies, LTspiceAC])
LTspiceTrace2.label       = 'LTspice: A1 P anti-series cascode stage'

print("\n**********************************************************************")
print("* Check the AC performance of dual stage with LTspice                *")
print("**********************************************************************\n")

netlist = SpiceEditor('cir/twoStageLTspice_pn.cir')
netlist.set_parameters(Lp=Le, Wp=W_finger, Mp=2*Mf, fmin=f_min, fmax=f_max, R_i=Ri, IDp=-2*ID, Wn=5e-6, Ln=0.18e-6, Mn=50, IDn=IDp)
netlist.add_instructions(".ac dec 20 10 100k")

fi = open('cir/twoStageLTspice_ac.cir', 'w')
fi.writelines(netlist.netlist)
fi.close()

runLTspice('cir/twoStageLTspice_ac.cir')

LTC                      = SimRunner(output_folder = './cir')
LTR                      = RawRead('cir/twoStageLTspice_ac.raw')
LTspiceACFrequencies     = LTR.get_trace('frequency').data
LTspiceAC                = np.abs(LTR.get_trace('V(out)').data)
LTspiceTrace3             = trace([LTspiceACFrequencies, LTspiceAC])
LTspiceTrace3.label       = 'LTspice: A1 P anti-series CS stage + N CS stage'

traceDict={1:LTspiceTrace1, 2:LTspiceTrace2, 3:LTspiceTrace3}
figMag = plot("LTspiceMag", "Magnitude of the small-signal voltage tansfer", "log", traceDict, xName = "frequency", xUnits="Hz", yName="Vout/Vs", show=True)