#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec  6 12:00:30 2023

@author: anton

Copy 'extras.py' from ~/SLiCAP/examples/MOS_EKV_BSIM/extras.py
Copy 'MOS_OP_I.cir' from ~/SLiCAP/examples/MOS_EKV_BSIM/cir/MOS_OP_I.cir

- Import MOS design data (W, L, M, ID) from noiseWLI.py
- import the frequency limits 'f_min' and 'f_max' from noiseWLI.py and
  convert them to standard python floats.
- Define the library and the device
- Define the operating voltages 'VB', 'VS', and 'VD'
- Plot the input noise spectrum obtained from the EKV model and the BSIM model
"""
from SLiCAP import *
from extras import *
from noiseWLI_n import W_finger, ID, Mf, Le, f_min, f_max

# type conversion
fmin = float(f_min)
fmax = float(f_max)

# Define the library and the device
LIB = '.lib lib/log018.l TT'   # use this CMOS BSIM library with EKV CMOS-1.lib
DEV = 'nch'
device = MOS('M1', LIB, DEV, W_finger, Le, Mf)

# Define the operating voltages 'VB', 'VS', and 'VD'
VS = 0
VB = 0
VD = 0.5

fm = np.sqrt(fmin * fmax)

# Plot the input noise spectrum obtained from the EKV model and the BSIM model
Vinoise_plot, FT, FL = plotSvinoise(device, ID, VD, VS, VB, fmin, fmax, 20, EKVlib = 'CMOS18-1.lib', fileName='NMOS_EKV_BSIM')

# print operating point information
VG = 0
device.getOPid(ID, VD, VS, VB, fm)

VGS = device.params['v(vgs)']
VDS = VD - VS

###############################################################################
# HTML Report                                                                 #
###############################################################################

ini.htmlIndex = 'index.html'
htmlPage('NMOS   EKV versus BSIM model')
head2html('Device geometry and operating point')
text2html('The selected NMOS has the following geometry:')
text2html('Channel length $L=%s\\,\\mu$m,'%(sp.latex(sp.N(Le*1e6, 3))))
text2html('Finger width $W=%s\\,\\mu$m,'%(sp.latex(sp.N(W_finger*1e6, 3))))
text2html('Number of fingers $M=%s$,'%(sp.latex(sp.N(Mf))))
text2html('This NMOS operates in weak inversion in the forward saturation region.')
text2html('DC operating current $I_{DS}=%s\\,A$,'%(sp.latex(sp.N(ID,3))))
text2html('DC operating voltage $V_{DS}=%s\\,V$,'%(sp.latex(sp.N(VDS,3))))
head2html('Input-referred noise voltage density spectrum')
text2html('The figure below shows the input-referred noise voltage density spectrum in $\\frac{V^2}{Hz}$.')
fig2html(Vinoise_plot, 600)
text2html('The cut-off frequency $f_T$ equals $%s Hz.$'%(sp.latex(sp.N(FT, 3))))
text2html('The corner frequency of the flicker noise $f_{\\ell}$ equals $%s$ Hz.'%(sp.latex(sp.N(FL, 3))))
text2html('The ratio $\\frac{f_T}{f_{\\ell}}$ equals $%s$.'%(sp.latex(sp.N(FT/FL, 3))))