#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec  6 12:39:30 2023

@author: anton
"""

from SLiCAP import *
from specifications import V_noise
from checkModel import VGS, VDS
from PyLTSpice import Trace, RawRead, SpiceEditor, SimRunner
from noise2 import Ri
from noiseCissGm import N_var_dinA
from noiseWLI import onoisePlot, DinA, f, RMS_dinA, W_finger, ID, Mf, Le, f_min, f_max, IC

netlist = SpiceEditor('cir/noiseLTspice.cir')
netlist.set_parameters(L = Le, W = W_finger, M = Mf, ID = ID, VGS = VGS,
                       VDS = VDS, fmin = f_min, fmax = f_max, R_i = Ri)

fi = open('cir/noiseLTspice.cir', 'w')
fi.writelines(netlist.netlist)
fi.close()

runLTspice('cir/noiseLTspice.cir')

LTC                      = SimRunner(output_folder = './cir')
LTR                      = RawRead('cir/noiseLTspice.raw')
LTspiceOnoiseFrequencies = LTR.get_trace('frequency').data
LTspiceOnoiseSpectrum    = LTR.get_trace('V(onoise)').data**2
func_dinA                = sp.lambdify(f, DinA)
dinA_weighted_spectrum   = LTspiceOnoiseSpectrum * func_dinA(LTspiceOnoiseFrequencies)
dinA_RMS_LTspice         = np.sqrt(np.trapz(dinA_weighted_spectrum, LTspiceOnoiseFrequencies))
LTspiceTrace             = trace([LTspiceOnoiseFrequencies, LTspiceOnoiseSpectrum])
LTspiceTrace.label       = 'LTspice'
LTspiceNoiseDict         = {'LTspice': LTspiceTrace}

addTraces(onoisePlot, LTspiceNoiseDict)
onoisePlot.plot()

print('\nSpecification and verification data:`\n')
print("Target DIN A weighted RMS output noise     :", str(sp.N(V_noise, 3)))
print("g_m, c_iss DIN A weighted RMS output noise :", str(sp.N(sp.sqrt(N_var_dinA), 3)))
print("W, L, ID DIN A weighted RMS output noise   :", str(sp.N(RMS_dinA, 3)))
print("LTspice DIN A weighted RMS output noise    :", str(sp.N(dinA_RMS_LTspice, 3)))
print('\nDesign data:`\n')
print("NMOS finger width                          :", sp.N(W_finger, 3))
print("NMOS number of fingers                     :", sp.N(Mf, 3))
print("NMOS channel length                        :", sp.N(Le,3))
print("NMOS DC drain current                      :", sp.N(ID, 3))
print("NMOS VGS                                   :", sp.N(VGS, 3))
print("NMOS VDS                                   :", sp.N(VDS, 3))
print("NMOS inversion coefficient                 :", sp.N(IC, 3))

###############################################################################
# HTML Report                                                                 #
###############################################################################

ini.htmlIndex = 'index.html'
htmlPage('EKV versus BSIM DIN A weighted RMS output noise')
head2html('LTspice test bench')
text2html('Below the LTspice test circuit.')
img2html('noiseLTspice.svg', 1000)
head2html('Noise spectra')
text2html('Below the noise spectra evaluated with the EKV model (red) and with the BSIM model (blue).')
fig2html(onoisePlot, 600)
head2html('DIN A weighted RMS output noise')
text2html('SLiCAP EKV calculated: $%s\\, \\mu$V.'%(sp.latex(sp.N(RMS_dinA * 1e6, 3))))
text2html('LTspice BSIM simulated: $%s\\, \\mu$V.'%(sp.latex(sp.N(dinA_RMS_LTspice * 1e6, 3))))