#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec  6 12:39:30 2023

@author: anton
"""

from SLiCAP import *
from specifications import V_noise
from noise2 import Ri, IL_fb_max
from PyLTSpice import Trace, RawRead, SpiceEditor, SimRunner
from noiseCissGm_p import N_var_dinA
from noiseWLI_p import onoisePlotP, DinA, RMS_dinA, W_finger, Mf, Le, f_min, f_max, ID
from oneStage_p import IDp

print("Specified maximum DIN A weighted RMS noise:", sp.N(V_noise, 3))

func_dinA                = sp.lambdify(ini.frequency, DinA)

print("\n**********************************************************************")
print("* Check the noise performance of single CS stage with LTspice        *")
print("**********************************************************************\n")

netlist = SpiceEditor('cir/oneStageLTspice_p.cir')
netlist.set_parameters(Lp=Le, Wp=W_finger, Mp=Mf, fmin=f_min, fmax=f_max, R_i=Ri, ID=IDp)
netlist.add_instructions(".noise V(out) V4 dec 20 {fmin} {fmax}")
#netlist.add_instructions(".tran 0 10m 9m")
#netlist.add_instructions(".ac dec 20 10 100k")

fi = open('cir/oneStageLTspice_noise.cir', 'w')
fi.writelines(netlist.netlist)
fi.close()

runLTspice('cir/oneStageLTspice_noise.cir')

LTC                      = SimRunner(output_folder = './cir')
LTR                      = RawRead('cir/oneStageLTspice_noise.raw')
LTspiceOnoiseFrequencies = LTR.get_trace('frequency').data
LTspiceOnoiseSpectrum    = LTR.get_trace('V(onoise)').data**2
dinA_weighted_spectrum   = LTspiceOnoiseSpectrum * func_dinA(LTspiceOnoiseFrequencies)
dinA_RMS_LTspiceCS       = np.sqrt(np.trapz(dinA_weighted_spectrum, LTspiceOnoiseFrequencies))
LTspiceTrace1            = trace([LTspiceOnoiseFrequencies, LTspiceOnoiseSpectrum])
LTspiceTrace1.label      = 'LTspice: A1 P anti-series CS Stage'

print("DIN A weighted RMS noise CS stage:", sp.N(dinA_RMS_LTspiceCS, 3))

print("\n**********************************************************************")
print("* Check the noise performance of single cascode stage with LTspice   *")
print("**********************************************************************\n")

netlist = SpiceEditor('cir/oneCascodeStageLTspice_p.cir')
netlist.set_parameters(Lp=Le, Wp=W_finger, Mp=Mf, fmin=f_min, fmax=f_max, R_i=Ri, ID=IDp)
netlist.add_instructions(".noise V(out) V4 dec 20 {fmin} {fmax}")

fi = open('cir/oneCascodeStageLTspice_noise.cir', 'w')
fi.writelines(netlist.netlist)
fi.close()

runLTspice('cir/oneCascodeStageLTspice_noise.cir')

LTC                      = SimRunner(output_folder = './cir')
LTR                      = RawRead('cir/oneCascodeStageLTspice_noise.raw')
LTspiceOnoiseFrequencies = LTR.get_trace('frequency').data
LTspiceOnoiseSpectrum    = LTR.get_trace('V(onoise)').data**2
dinA_weighted_spectrum   = LTspiceOnoiseSpectrum * func_dinA(LTspiceOnoiseFrequencies)
dinA_RMS_LTspiceCascode  = np.sqrt(np.trapz(dinA_weighted_spectrum, LTspiceOnoiseFrequencies))
LTspiceTrace2            = trace([LTspiceOnoiseFrequencies, LTspiceOnoiseSpectrum])
LTspiceTrace2.label      = 'LTspice: A1 P anti-series cascode stage'

print("DIN A weighted RMS noise cascode stage:", sp.N(dinA_RMS_LTspiceCascode,3))

print("\n**********************************************************************")
print("* Check the noise performance of dual stage with LTspice              *")
print("**********************************************************************\n")

netlist = SpiceEditor('cir/twoStageLTspice_pn.cir')
netlist.set_parameters(Lp=Le, Wp=W_finger, Mp=2*Mf, fmin=f_min, fmax=f_max, R_i=Ri, IDp=-2*ID, Wn=5e-6, Ln=0.18e-6, Mn=50, IDn=IDp)
netlist.add_instructions(".noise V(out) V4 dec 20 {fmin} {fmax}")

fi = open('cir/twoStageLTspice_noise.cir', 'w')
fi.writelines(netlist.netlist)
fi.close()

runLTspice('cir/twoStageLTspice_noise.cir')

LTC                      = SimRunner(output_folder = './cir')
LTR                      = RawRead('cir/twoStageLTspice_noise.raw')
LTspiceOnoiseFrequencies = LTR.get_trace('frequency').data
LTspiceOnoiseSpectrum    = LTR.get_trace('V(onoise)').data**2
dinA_weighted_spectrum   = LTspiceOnoiseSpectrum * func_dinA(LTspiceOnoiseFrequencies)
dinA_RMS_LTspiceTwoStage = np.sqrt(np.trapz(dinA_weighted_spectrum, LTspiceOnoiseFrequencies))
LTspiceTrace3            = trace([LTspiceOnoiseFrequencies, LTspiceOnoiseSpectrum])
LTspiceTrace3.label      = 'LTspice: A1 P anti-series CS stage + N CS stage'

print("DIN A weighted noise two-stage:", sp.N(dinA_RMS_LTspiceTwoStage, 3))

traceDict={1:LTspiceTrace1, 2:LTspiceTrace2, 3:LTspiceTrace3}
addTraces(onoisePlotP, traceDict)
onoisePlotP.plot()