#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Dec 18 13:05:55 2023

@author: anton
"""
from SLiCAP import *
from noise2 import IL_fb_max
from noiseWLI_p import W_finger, Mf, Le, f_min, f_max

print("\n**********************************************************************")
print("* Check bandwidth and midband accuracy of single-stage PMOS solution *")
print("**********************************************************************\n")

# define the stage parameters
# Define test frequency for determination of small-signal operating point parameters
f        = sp.sqrt(f_min*f_max)
# Define BSIM LIB
LIB      = '.lib lib/log018.l TT'

# first stage. Import design data from noise W, L, I design, but change the current
IDp = 0
while IDp <= IL_fb_max/2:
    IDp = float(input("Enter the DC operating current of the NMOS in this stage (> %s):\n>>> "%(str(sp.N(IL_fb_max/2, 3)))))

W1   = W_finger
L1   = Le
M1   = Mf
VD1  = 0.4
MOS1 = MOS('M1', LIB, 'pch', W1, L1, M1)
MOS1.getOPid(-IDp, VD1, 0.9, 0.9, f)

fileName = "oneStage"

i1 = instruction()
i1.setCircuit(fileName + ".cir")
specs = csv2specs('A1specs.csv')
specs2circuit(specs, i1.circuit)
gm   = MOS1.parDefs[sp.Symbol('gm_M1')]
cgs  = MOS1.parDefs[sp.Symbol('cgs_M1')]
go   = 1/MOS1.parDefs[sp.Symbol('go_M1')]
Ls   = i1.getParValue('L_s')
Rs   = i1.getParValue('R_s')
Rt   = i1.getParValue('R_t')
Ri   = i1.getParValue('R_i')
Ci   = i1.getParValue('C_i')

print("IDS1:", sp.N(MOS1.params['i(ids)'], 3))
print("VGS1:", sp.N(MOS1.params['v(vgs)'], 3))
print('g_m :', sp.N(gm, 3))
print('c_gg:', sp.N(cgs, 3))
print('r_o :', sp.N(go, 3))
print('L_s :', sp.N(Ls, 3))
print('R_s :', sp.N(Rs, 3))
print('R_t :', sp.N(Rt, 3))
print('R_i :', sp.N(Ri, 3))
print('C_i :', sp.N(Ci, 3))

# Use operating point information
i1.defPar('g_m', gm)
i1.defPar('c_gg', cgs)
i1.defPar('r_o', go)
i1.setSimType("numeric")
i1.setDataType("laplace")
i1.setSource("V1")
i1.setDetector("V_out")
i1.setLGref("G1")

# Remove the local feedback around the loop gain reference
#i1.defPar("C_gd1", 0)
i1.setGainType("asymptotic")
AGresult = i1.execute()

i1.setGainType("loopgain")
LGresult = i1.execute()

i1.setGainType("servo")
SVresult = i1.execute()

# For evaluation of the gain with cdg_M1 included uncomment the next line
i1.defPar('C_gd1', MOS1.parDefs[sp.Symbol('cdg_M1')])

i1.setGainType("direct")
DIresult = i1.execute()

i1.setGainType("gain")
GAresult = i1.execute()

results = [AGresult, LGresult, SVresult, GAresult, DIresult]
fbMag = plotSweep("NMOS_mag_1", "Feedback Model Magnitude single-stage NMOS", results, 0.01, 1e9, 200, show=True)
fbPhs = plotSweep("NMOS_phase_1", "Feedback Model Phase single-stage NMOS", results, 0.01, 1e9, 200, funcType="phase", show=True)

i1.setDataType('pz')
i1.setGainType('loopgain')
LGpz  = i1.execute()

listPZ(LGpz)

servoInfo = findServoBandwidth(LGresult.laplace)

if servoInfo['hpf'] <= f_min and servoInfo['lpf'] > f_max:
    print("Bandwidth single-stage solution is OK!")
    print("Frequency range %s ... %s Hz."%(str(sp.N(servoInfo['hpf'], 3)), str(sp.N(servoInfo['lpf'], 3))))
    print("Magnitude of midband loop gain single-stage solution:", str(sp.N(servoInfo['mbv'], 3)))
    print("Maximum loop gain at %s Hz"%(str(sp.N(servoInfo['mbf'], 3))))

###############################################################################
# HTML Report                                                                 #
###############################################################################
