#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Dec 18 13:05:55 2023

@author: anton
"""
from SLiCAP import *
from noiseWLI import W_finger, ID, Mf, Le, f_min, f_max

# define the stage parameters
# Define test frequency for determination of small-signal operating point parameters
f        = sp.sqrt(f_min*f_max)
# Define BSIM LIB
LIB      = '.lib lib/log018.l TT'

# first stage. Import design data from noise W, L, I design
ID1  = ID
W1   = W_finger
L1   = Le
M1   = Mf
VD1  = 0.45
MOS1 = MOS('M1', LIB, 'nch', W1, L1, M1)
MOS1.getOPid(ID1, VD1, 0, 0, f)

print("IDS1:", MOS1.params['i(ids)'])
print("VGS1:", MOS1.params['v(vgs)'])

# second stage. Define stage with operating current larger than peak value of
#               through the feedback network
ID2  = 25e-6
W2   = 1e-6
L2   = 0.18e-6
M2   = 5
VD2  = 0.45
MOS2 = MOS('M2', LIB, 'nch', W2, L2, M2)
MOS2.getOPid(ID2, VD2, 0, 0, f)

print("IDS2:", MOS2.params['i(ids)'])
print("VGS2:", MOS2.params['v(vgs)'])

fileName = "twoStage1"
i1 = instruction()
i1.setCircuit(fileName + ".cir")
specs = csv2specs('A1specs.csv')
specs2circuit(specs, i1.circuit)

# Use operating point information
i1.defPar('g_m1', MOS1.parDefs[sp.Symbol('gm_M1')])
i1.defPar('C_gs1', MOS1.parDefs[sp.Symbol('cgs_M1')])
i1.defPar('C_gd1', MOS1.parDefs[sp.Symbol('cdg_M1')])
i1.defPar('r_o1', 1/MOS1.parDefs[sp.Symbol('go_M1')])
i1.defPar('g_m2', MOS2.parDefs[sp.Symbol('gm_M2')])
i1.defPar('C_gs2', MOS2.parDefs[sp.Symbol('cgs_M2')])
i1.defPar('C_gd2', MOS2.parDefs[sp.Symbol('cdg_M2')])
i1.defPar('r_o2', 1/MOS2.parDefs[sp.Symbol('go_M2')])

i1.setSimType("numeric")
i1.setDataType("laplace")
i1.setSource("V1")
i1.setDetector("V_out")
i1.setLGref("G1")

# Remove the local feedback around the loop gain reference
#i1.defPar("C_gd1", 0)
i1.setGainType("asymptotic")
AGresult = i1.execute()

i1.setGainType("loopgain")
LGresult = i1.execute()

i1.setGainType("servo")
SVresult = i1.execute()

# For evaluation of the gain with cdg_M1 included uncomment the next line
i1.defPar('C_gd1', MOS1.parDefs[sp.Symbol('cdg_M1')])

i1.setGainType("direct")
DIresult = i1.execute()

i1.setGainType("gain")
GAresult = i1.execute()

results = [AGresult, LGresult, SVresult, GAresult, DIresult]
fbMag = plotSweep("Feedback Model Magnitude", "A1", results, 0.01, 1e9, 200, show=True)
fbPhs = plotSweep("Feedback Model Phase", "A1", results, 0.01, 1e9, 200, funcType="phase", show=True)

i1.setDataType('pz')
GApz  = i1.execute()

listPZ(GApz)

###############################################################################
# HTML Report                                                                 #
###############################################################################
